@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: module.cpp 1015 2011-04-27 10:19:23Z oconrad $
+ * Version $Id: module.cpp 1687 2013-05-14 11:38:28Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -65,6 +65,8 @@
 //---------------------------------------------------------
 #include "module.h"
 
+#include "data_manager.h"
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -80,17 +82,13 @@ CSG_Module::CSG_Module(void)
 	m_bError_Ignore	= false;
 	m_bExecutes		= false;
 
-	m_Garbage		= NULL;
-	m_nGarbage		= 0;
-
 	m_pParameters	= NULL;
 	m_npParameters	= 0;
 
 	Parameters.Create(this, SG_T(""), SG_T(""));
 	Parameters.Set_Callback_On_Parameter_Changed(&_On_Parameter_Changed);
 
-	Set_Managed			(false);
-	Set_Show_Progress	(true);
+	Set_Show_Progress(true);
 }
 
 //---------------------------------------------------------
@@ -122,13 +120,6 @@ void CSG_Module::Destroy(void)
 	m_bError_Ignore	= false;
 
 	History_Supplement.Destroy();
-
-	Garbage_Clear();
-
-	if( m_bManaged )
-	{
-		SG_UI_Process_Set_Okay();
-	}
 }
 
 
@@ -139,54 +130,36 @@ void CSG_Module::Destroy(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CSG_Module::Set_Name(const SG_Char *String)
+void CSG_Module::Set_Name(const CSG_String &String)
 {
 	Parameters.Set_Name(String);
 }
 
-const SG_Char * CSG_Module::Get_Name(void)
+const CSG_String & CSG_Module::Get_Name(void)
 {
 	return( Parameters.Get_Name() );
 }
 
 //---------------------------------------------------------
-void CSG_Module::Set_Description(const SG_Char *String)
+void CSG_Module::Set_Description(const CSG_String &String)
 {
 	Parameters.Set_Description(String);
 }
 
-const SG_Char * CSG_Module::Get_Description(void)
+const CSG_String & CSG_Module::Get_Description(void)
 {
 	return( Parameters.Get_Description() );
 }
 
 //---------------------------------------------------------
-void CSG_Module::Set_Author(const SG_Char *String)
+void CSG_Module::Set_Author(const CSG_String &String)
 {
-	if( String )
-	{
-		m_Author	= String;
-	}
-	else
-	{
-		m_Author.Clear();
-	}
+	m_Author	= String;
 }
 
-const SG_Char * CSG_Module::Get_Author(void)
+const CSG_String & CSG_Module::Get_Author(void)
 {
-	return( m_Author.c_str() );
-}
-
-//---------------------------------------------------------
-void CSG_Module::Set_Translation(CSG_Translator &Translator)
-{
-	Parameters.Set_Translation(Translator);
-
-	for(int i=0; i<m_npParameters; i++)
-	{
-		m_pParameters[i]->Set_Translation(Translator);
-	}
+	return( m_Author );
 }
 
 
@@ -199,57 +172,69 @@ void CSG_Module::Set_Translation(CSG_Translator &Translator)
 //---------------------------------------------------------
 bool CSG_Module::Execute(void)
 {
+	if( m_bExecutes )
+	{
+		return( false );
+	}
+
+	m_bExecutes	= true;
+
+	Destroy();
+
 	bool	bResult	= false;
 
-	if( m_bExecutes == false )
+	//-----------------------------------------------------
+	if( !Parameters.DataObjects_Create() )
 	{
-		m_bExecutes		= true;
-
-		if( Parameters.DataObjects_Check() )
-		{
-			Destroy();
-
-			Parameters.DataObjects_Create();
-			Parameters.Msg_String(false);
+		Message_Dlg(_TL("could not initialize data objects"));
+	}
+	else
+	{
+		Parameters.Msg_String(false);
 
 ///////////////////////////////////////////////////////////
-#if !defined(_DEBUG) && defined(_SAGA_VC)
+#if !defined(_DEBUG) && defined(_SAGA_VC) && defined(WXWIN_28)
 #define _MODULE_EXCEPTION
-__try
-{
+		__try
+		{
 #endif
 ///////////////////////////////////////////////////////////
 
-			if( (bResult = On_Execute()) == true )
-			{
-				_Set_Output_History();
-			}
-
-			if( !Process_Get_Okay(false) )
-			{
-				SG_UI_Msg_Add(LNG("[MSG] Execution has been stopped by user!"), true);
-			}
+			bResult	= On_Execute();
 
 ///////////////////////////////////////////////////////////
 #ifdef _MODULE_EXCEPTION
-}	// try
-__except(1)
-{
-	Message_Add(LNG("[ERR] Module caused access violation!"));
-	Message_Dlg(LNG("[ERR] Module caused access violation!"));
-	bResult	= false;
-}	// except(1)
+		}	// try
+		__except(1)
+		{
+			Message_Dlg(SG_T("Module caused access violation!"));
+		}	// except(1)
 #endif
 ///////////////////////////////////////////////////////////
 
-			Destroy();
-
-			_Synchronize_DataObjects();
+		if( bResult )
+		{
+			_Set_Output_History();
 		}
 
-		m_bExecutes		= false;
+		if( !Process_Get_Okay(false) )
+		{
+			SG_UI_Msg_Add(_TL("Execution has been stopped by user!"), true);
+
+			bResult	= false;
+		}
+
+		_Synchronize_DataObjects();
 	}
 
+	//-----------------------------------------------------
+	Destroy();
+
+	SG_UI_Process_Set_Ready();
+	SG_UI_Process_Set_Okay();
+
+	m_bExecutes	= false;
+
 	return( bResult );
 }
 
@@ -330,13 +315,13 @@ bool CSG_Module::Get_Projection(CSG_Projection &Projection)	const
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CSG_Module::Set_Managed(bool bOn)
+void CSG_Module::Set_Manager(class CSG_Data_Manager *pManager)
 {
-	m_bManaged	= Parameters.m_bManaged	= bOn;
+	Parameters.Set_Manager(pManager);
 
 	for(int i=0; i<m_npParameters; i++)
 	{
-		m_pParameters[i]->m_bManaged	= bOn;
+		m_pParameters[i]->Set_Manager(pManager);
 	}
 }
 
@@ -396,7 +381,7 @@ int CSG_Module::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CSG_Parameters * CSG_Module::Add_Parameters(const SG_Char *Identifier, const SG_Char *Name, const SG_Char *Description)
+CSG_Parameters * CSG_Module::Add_Parameters(const CSG_String &Identifier, const CSG_String &Name, const CSG_String &Description)
 {
 	CSG_Parameters	*pParameters;
 
@@ -410,7 +395,7 @@ CSG_Parameters * CSG_Module::Add_Parameters(const SG_Char *Identifier, const SG_
 }
 
 //---------------------------------------------------------
-CSG_Parameters * CSG_Module::Get_Parameters(const SG_Char *Identifier)
+CSG_Parameters * CSG_Module::Get_Parameters(const CSG_String &Identifier)
 {
 	CSG_String	sIdentifier(Identifier);
 
@@ -426,11 +411,13 @@ CSG_Parameters * CSG_Module::Get_Parameters(const SG_Char *Identifier)
 }
 
 //---------------------------------------------------------
-bool CSG_Module::Dlg_Parameters(const SG_Char *Identifier)
+bool CSG_Module::Dlg_Parameters(const CSG_String &Identifier)
 {
-	if( !m_bManaged || Dlg_Parameters(Get_Parameters(Identifier), Get_Name()) )
+	CSG_Parameters	*pParameters	= Get_Parameters(Identifier);
+
+	if( pParameters && (!pParameters->is_Managed() || Dlg_Parameters(pParameters, Get_Name())) )
 	{
-		Get_Parameters(Identifier)->Set_History(History_Supplement);
+		pParameters->Set_History(History_Supplement);
 
 		return( true );
 	}
@@ -439,25 +426,11 @@ bool CSG_Module::Dlg_Parameters(const SG_Char *Identifier)
 }
 
 //---------------------------------------------------------
-bool CSG_Module::Dlg_Parameters(CSG_Parameters *pParameters, const SG_Char *Caption)
+bool CSG_Module::Dlg_Parameters(CSG_Parameters *pParameters, const CSG_String &Caption)
 {
 	return( pParameters ? SG_UI_Dlg_Parameters(pParameters, Caption) : false );
 }
 
-//---------------------------------------------------------
-#ifdef _SAGA_UNICODE
-
-CSG_Parameters *	CSG_Module::Add_Parameters(const char *Identifier, const SG_Char *Name, const SG_Char *Description)
-{	return( Add_Parameters(CSG_String(Identifier), Name, Description) );	}
-
-CSG_Parameters *	CSG_Module::Get_Parameters(const char *Identifier)
-{	return( Get_Parameters(CSG_String(Identifier)) );	}
-
-bool				CSG_Module::Dlg_Parameters(const char *Identifier)
-{	return( Dlg_Parameters(CSG_String(Identifier)) );	}
-
-#endif
-
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -472,7 +445,7 @@ bool CSG_Module::Process_Get_Okay(bool bBlink)
 }
 
 //---------------------------------------------------------
-void CSG_Module::Process_Set_Text(const SG_Char *Text)
+void CSG_Module::Process_Set_Text(const CSG_String &Text)
 {
 	SG_UI_Process_Set_Text(Text);
 }
@@ -503,19 +476,19 @@ bool CSG_Module::Stop_Execution(bool bDialog)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CSG_Module::Message_Dlg(const SG_Char *Text, const SG_Char *Caption)
+void CSG_Module::Message_Dlg(const CSG_String &Text, const SG_Char *Caption)
 {
-	SG_UI_Dlg_Message(Text, Caption && Caption[0] != '\0' ? Caption : Get_Name());
+	SG_UI_Dlg_Message(Text, Caption && Caption[0] != '\0' ? Caption : Get_Name().c_str());
 }
 
 //---------------------------------------------------------
-bool CSG_Module::Message_Dlg_Confirm(const SG_Char *Text, const SG_Char *Caption)
+bool CSG_Module::Message_Dlg_Confirm(const CSG_String &Text, const SG_Char *Caption)
 {
-	return( SG_UI_Dlg_Continue(Text, Caption && Caption[0] != '\0' ? Caption : Get_Name()) );
+	return( SG_UI_Dlg_Continue(Text, Caption && Caption[0] != '\0' ? Caption : Get_Name().c_str()) );
 }
 
 //---------------------------------------------------------
-void CSG_Module::Message_Add(const SG_Char *Text, bool bNewLine)
+void CSG_Module::Message_Add(const CSG_String &Text, bool bNewLine)
 {
 	SG_UI_Msg_Add_Execution(Text, bNewLine);
 }
@@ -526,21 +499,21 @@ bool CSG_Module::Error_Set(TSG_Module_Error Error_ID)
 	switch( Error_ID )
 	{
 	default:
-		return( Error_Set(LNG("[ERR] Unknown Error")) );
+		return( Error_Set(_TL("Unknown Error")) );
 	    
 	case MODULE_ERROR_Calculation:
-		return( Error_Set(LNG("[ERR] Calculation Error")) );
+		return( Error_Set(_TL("Calculation Error")) );
 	}
 }
 
 //---------------------------------------------------------
-bool CSG_Module::Error_Set(const SG_Char *Error_Text)
+bool CSG_Module::Error_Set(const CSG_String &Error_Text)
 {
 	SG_UI_Msg_Add_Error(Error_Text);
 
 	if( SG_UI_Process_Get_Okay(false) && !m_bError_Ignore )
 	{
-		switch( SG_UI_Dlg_Error(Error_Text, LNG("[ERR] Error: Continue anyway ?")) )
+		switch( SG_UI_Dlg_Error(Error_Text, _TL("Error: Continue anyway ?")) )
 		{
 		case 0: default:
 			SG_UI_Process_Set_Okay(false);
@@ -558,92 +531,19 @@ bool CSG_Module::Error_Set(const SG_Char *Error_Text)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//				DataObjects / Garbage					 //
-//														 //
-///////////////////////////////////////////////////////////
-
-//---------------------------------------------------------
-bool CSG_Module::_Garbage_Add_Item(CSG_Data_Object *pDataObject)
-{
-	if( pDataObject && !SG_UI_DataObject_Check(pDataObject, DATAOBJECT_TYPE_Undefined) )
-	{
-		for(int i=0; i<m_nGarbage; i++)
-		{
-			if( pDataObject == m_Garbage[i] )
-			{
-				return( true );
-			}
-		}
-
-		m_Garbage	= (CSG_Data_Object **)SG_Realloc(m_Garbage, (m_nGarbage + 1) * sizeof(CSG_Data_Object *));
-		m_Garbage[m_nGarbage++]	= pDataObject;
-
-		return( true );
-	}
-
-	return( false );
-}
-
-//---------------------------------------------------------
-CSG_Data_Object * CSG_Module::Garbage_Del_Item(int i, bool bFromListOnly)
-{
-	CSG_Data_Object	*pDataObject	= NULL;
-
-	if( i >= 0 && i < m_nGarbage )
-	{
-		if( bFromListOnly || !m_Garbage[i] )
-		{
-			pDataObject	= m_Garbage[i];
-		}
-		else if( m_Garbage[i] )
-		{
-			delete(m_Garbage[i]);
-		}
-
-		for(m_nGarbage--; i<m_nGarbage; i++)
-		{
-			m_Garbage[i]	= m_Garbage[i + 1];
-		}
-
-		m_Garbage	= (CSG_Data_Object **)SG_Realloc(m_Garbage, m_nGarbage * sizeof(CSG_Data_Object *));
-	}
-
-	return( pDataObject );
-}
-
-//---------------------------------------------------------
-void CSG_Module::Garbage_Clear(void)
-{
-	if( m_nGarbage > 0 )
-	{
-		for(int i=0; i<m_nGarbage; i++)
-		{
-			if( m_Garbage[i] )
-			{
-				delete(m_Garbage[i]);
-			}
-		}
-
-		SG_Free(m_Garbage);
-	}
-
-	m_nGarbage	= 0;
-	m_Garbage	= NULL;
-}
-
-
-///////////////////////////////////////////////////////////
-//														 //
-//				DataObjects / GUI Interaction			 //
+//			DataObjects / GUI Interaction				 //
 //														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 bool CSG_Module::DataObject_Add(CSG_Data_Object *pDataObject, bool bUpdate)
 {
-	return( m_bManaged && SG_UI_DataObject_Add(pDataObject, bUpdate)
-		? true : _Garbage_Add_Item(pDataObject)
-	);
+	if( Parameters.Get_Manager() )
+	{
+		Parameters.Get_Manager()->Add(pDataObject);
+	}
+
+	return( SG_UI_DataObject_Add(pDataObject, bUpdate) );
 }
 
 //---------------------------------------------------------
@@ -857,6 +757,272 @@ void CSG_Module::_Set_Output_History(void)
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//						Summary							 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+void _Add_XML(CSG_MetaData *pParent, CSG_Parameter *pParameter, CSG_String ID = "")
+{
+	if( !ID.is_Empty() )	ID	+= "_";	ID	+= pParameter->Get_Identifier();
+
+	CSG_MetaData	*pItem	= pParent->Add_Child(SG_XML_PARAM);
+
+	pItem->Add_Property(SG_XML_PARAM_ATT_NAME , pParameter->Get_Name());
+	pItem->Add_Property(SG_XML_PARAM_ATT_CLASS,	pParameter->is_Input() ? "input" : pParameter->is_Output() ? "output" : "option");
+
+	pItem->Add_Child(SG_XML_PARAM_IDENT, ID);
+	pItem->Add_Child(SG_XML_PARAM_TYPE , pParameter->Get_Type_Name().Make_Lower());
+	pItem->Add_Child(SG_XML_PARAM_MAND , pParameter->is_Optional() ? SG_T("false") : SG_T("true"));
+	pItem->Add_Child(SG_XML_DESCRIPTION, pParameter->Get_Description());
+
+	switch( pParameter->Get_Type() )
+	{
+	//-----------------------------------------------------
+	case PARAMETER_TYPE_Choice:
+		{
+			CSG_MetaData	*pChild	= pItem->Add_Child(SG_XML_PARAM_LIST);
+
+			for(int i=0; i<pParameter->asChoice()->Get_Count(); i++)
+			{
+				pChild->Add_Child(SG_XML_PARAM_ITEM, pParameter->asChoice()->Get_Item(i));
+			}
+		}
+		break;
+
+	//-----------------------------------------------------
+	case PARAMETER_TYPE_FixedTable:
+		{
+			CSG_MetaData	*pChild	= pItem->Add_Child(SG_XML_PARAM_TABLE);
+
+			for(int i=0; i<pParameter->asTable()->Get_Field_Count(); i++)
+			{
+				CSG_MetaData	*pField	= pChild->Add_Child(SG_XML_PARAM_FIELD);
+
+				pField->Add_Property(SG_XML_PARAM_FIELD_ATT_NAME, pParameter->asTable()->Get_Field_Name(i));
+				pField->Add_Property(SG_XML_PARAM_FIELD_ATT_TYPE, SG_Data_Type_Get_Name(pParameter->asTable()->Get_Field_Type(i)));
+			}
+		}
+		break;
+
+	//-----------------------------------------------------
+	case PARAMETER_TYPE_Int:
+	case PARAMETER_TYPE_Double:
+	case PARAMETER_TYPE_Degree:
+		if( pParameter->asValue()->has_Minimum() )	pItem->Add_Child(SG_XML_PARAM_MIN, pParameter->asValue()->Get_Minimum());
+		if( pParameter->asValue()->has_Maximum() )	pItem->Add_Child(SG_XML_PARAM_MAX, pParameter->asValue()->Get_Maximum());
+		break;
+
+	//-----------------------------------------------------
+	case PARAMETER_TYPE_Parameters:
+		{
+			for(int i=0; i<pParameter->asParameters()->Get_Count(); i++)
+			{
+				_Add_XML(pItem, pParameter->asParameters()->Get_Parameter(i), ID);
+			}
+		}
+	}
+}
+
+//---------------------------------------------------------
+#define SUMMARY_ADD_STR(label, value)	s += CSG_String::Format(SG_T("<tr><td valign=\"top\">%s</td><td valign=\"top\">%s</td></tr>"), label, value)
+#define SUMMARY_ADD_INT(label, value)	s += CSG_String::Format(SG_T("<tr><td valign=\"top\">%s</td><td valign=\"top\">%d</td></tr>"), label, value)
+
+//---------------------------------------------------------
+CSG_String CSG_Module::Get_Summary(bool bParameters, const CSG_String &Menu, const CSG_String &Description, bool bXML)
+{
+	int			i;
+	CSG_String	s;
+
+	//-----------------------------------------------------
+	if( bXML )
+	{
+		CSG_MetaData	m;
+
+		m.Set_Name    (SG_XML_MODULE);
+		m.Add_Property(SG_XML_MODULE_ATT_NAME  , Get_Name       ());
+		m.Add_Property(SG_XML_MODULE_ATT_ID    , Get_ID         ());
+		m.Add_Property(SG_XML_MODULE_ATT_AUTHOR, Get_Author     ());
+		m.Add_Child   (SG_XML_DESCRIPTION      , Get_Description());
+		m.Add_Child   (SG_XML_MENU             , Get_MenuPath   ());
+		m.Add_Child   (SG_XML_SPEC_ATT_GRID    , is_Grid        () ? SG_T("true") : SG_T("false"));
+		m.Add_Child   (SG_XML_SPEC_ATT_GRID    , is_Interactive () ? SG_T("true") : SG_T("false"));
+
+	//	CSG_MetaData	*pChild	= m.Add_Child(SG_XML_SPECIFICATION);
+	//	pChild->Add_Property(SG_XML_SPEC_ATT_GRID    , is_Grid        () ? SG_T("true") : SG_T("false"));
+	//	pChild->Add_Property(SG_XML_SPEC_ATT_INTERA  , is_Interactive () ? SG_T("true") : SG_T("false"));
+
+		if( bParameters )
+		{
+			for(i=0; i<Parameters.Get_Count(); i++)
+			{
+				if( Parameters(i)->is_Input() )
+ 				{
+					_Add_XML(&m, Parameters(i));
+				}
+			}
+
+			for(i=0; i<Parameters.Get_Count(); i++)
+			{
+				if( Parameters(i)->is_Output() )
+ 				{
+					_Add_XML(&m, Parameters(i));
+				}
+			}
+
+			for(i=0; i<Parameters.Get_Count(); i++)
+			{
+	 			if( Parameters(i)->is_Option()
+				&&  Parameters(i)->Get_Type() != PARAMETER_TYPE_Node
+				&&  Parameters(i)->Get_Type() != PARAMETER_TYPE_Grid_System )
+ 				{
+					_Add_XML(&m, Parameters(i));
+				}
+			}
+		}
+
+		s	= m.asText(1);
+	}
+
+	//-----------------------------------------------------
+	else
+	{
+		s	+= CSG_String::Format(SG_T("<b>%s</b><table border=\"0\">"), _TL("Module"));
+
+		SUMMARY_ADD_STR(_TL("Name"  ), Get_Name  ().c_str());
+		SUMMARY_ADD_INT(_TL("ID"    ), Get_ID    ());
+		SUMMARY_ADD_STR(_TL("Author"), Get_Author().c_str());
+
+		if( is_Interactive() && is_Grid() )
+		{
+			SUMMARY_ADD_STR(_TL("Specification"), CSG_String::Format(SG_T("%s, %s"), _TL("grid"), _TL("interactive")).c_str() );
+		}
+		else if( is_Interactive() )
+		{
+			SUMMARY_ADD_STR(_TL("Specification"), _TL("interactive"));
+		}
+		else if( is_Grid() )
+		{
+			SUMMARY_ADD_STR(_TL("Specification"), _TL("grid"));
+		}
+
+		if( Menu.Length() > 0 )
+		{
+			CSG_String	sMenu(Menu);
+
+			sMenu.Replace(SG_T("|"), SG_T(" <b>></b> "));
+
+			SUMMARY_ADD_STR(_TL("Menu"  ), sMenu.c_str());
+		}
+
+		s	+= SG_T("</table><hr>");
+
+		//-------------------------------------------------
+		s	+= CSG_String::Format(SG_T("<b>%s</b><br>"), _TL("Description"));
+
+		s	+= Description.Length() > 0 ? Description : Get_Description();
+
+		//-------------------------------------------------
+		if( bParameters )
+		{
+			bool	bFirst, bOptionals	= false;
+
+			s	+= CSG_String::Format(SG_T("<hr><b>%s</b><br>"), _TL("Parameters"));
+			s	+= CSG_String::Format(SG_T("<table border=\"1\" width=\"100%%\" valign=\"top\" cellpadding=\"5\" rules=\"all\"><tr><th>%s</th><th>%s</th><th>%s</th><th>%s</th><th>%s</th></tr>\n"),
+					_TL("Name"), _TL("Type"), _TL("Identifier"), _TL("Description"), _TL("Constraints")
+				);
+
+			//---------------------------------------------
+			for(i=0, bFirst=true; i<Parameters.Get_Count(); i++)
+			{
+				CSG_Parameter	*pParameter	= Parameters(i);
+
+				if( pParameter->is_Input() )
+				{
+					if( bFirst )
+					{
+						bFirst	= false;
+						s	+= CSG_String::Format(SG_T("<tr><th colspan=\"5\">%s</th></tr>"), _TL("Input"));
+					}
+
+					s	+= CSG_String::Format(SG_T("<tr><td>%s%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr>"),
+						pParameter->Get_Name(),
+						pParameter->is_Optional() ? SG_T(" (*)") : SG_T(" "),
+						pParameter->Get_Description(PARAMETER_DESCRIPTION_TYPE).c_str(),
+						pParameter->Get_Identifier(),
+						pParameter->Get_Description(),
+						pParameter->Get_Description(PARAMETER_DESCRIPTION_PROPERTIES).c_str()
+					);
+				}
+			}
+
+			//---------------------------------------------
+			for(i=0, bFirst=true; i<Parameters.Get_Count(); i++)
+			{
+				CSG_Parameter	*pParameter	= Parameters(i);
+
+				if( pParameter->is_Output() )
+				{
+					if( bFirst )
+					{
+						bFirst	= false;
+						s	+= CSG_String::Format(SG_T("<tr><th colspan=\"5\">%s</th></tr>"), _TL("Output"));
+					}
+
+					s	+= CSG_String::Format(SG_T("<tr><td>%s%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr>"),
+						pParameter->Get_Name(),
+						pParameter->is_Optional() ? SG_T(" (*)") : SG_T(""),
+						pParameter->Get_Description(PARAMETER_DESCRIPTION_TYPE).c_str(),
+						pParameter->Get_Identifier(),
+						pParameter->Get_Description(),
+						pParameter->Get_Description(PARAMETER_DESCRIPTION_PROPERTIES).c_str()
+					);
+				}
+			}
+
+			//---------------------------------------------
+			for(i=0, bFirst=true; i<Parameters.Get_Count(); i++)
+			{
+				CSG_Parameter	*pParameter	= Parameters(i);
+
+				if( pParameter->is_Option() && pParameter->Get_Type() != PARAMETER_TYPE_Grid_System )
+				{
+					if( bFirst )
+					{
+						bFirst	= false;
+						s	+= CSG_String::Format(SG_T("<tr><th colspan=\"5\">%s</th></tr>"), _TL("Options"));
+					}
+
+					s	+= CSG_String::Format(SG_T("<tr><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr>"),
+						pParameter->Get_Name(),
+						pParameter->Get_Description(PARAMETER_DESCRIPTION_TYPE).c_str(),
+						pParameter->Get_Identifier(),
+						pParameter->Get_Description(),
+						pParameter->Get_Description(PARAMETER_DESCRIPTION_PROPERTIES).c_str()
+					);
+				}
+				else if( pParameter->is_Optional() )
+				{
+					bOptionals	= true;
+				}
+			}
+
+			//---------------------------------------------
+			s	+= SG_T("</table>");
+
+			if( bOptionals )
+			{
+				s	+= CSG_String::Format(SG_T("(*) <i>%s</i>"), _TL("optional"));
+			}
+		}
+	}
+
+	return( s );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
